"""
Copyright (c) 2025, Oracle and/or its affiliates.
Licensed under the Universal Permissive License v1.0 as shown at
https://oss.oracle.com/licenses/upl.
"""

import os
from logging import Logger
from typing import Annotated, List

import oci
from fastmcp import FastMCP
from oracle.oci_object_storage_mcp_server.models import (
    Bucket,
    BucketSummary,
    ListObjects,
    ObjectSummary,
    ObjectVersionCollection,
    map_bucket,
    map_bucket_summary,
    map_object_summary,
    map_object_version_summary,
)

from . import __project__, __version__

logger = Logger(__name__, level="INFO")

mcp = FastMCP(name=__project__)


def get_object_storage_client():
    config = oci.config.from_file(
        file_location=os.getenv("OCI_CONFIG_FILE", oci.config.DEFAULT_LOCATION),
        profile_name=os.getenv("OCI_CONFIG_PROFILE", oci.config.DEFAULT_PROFILE),
    )
    user_agent_name = __project__.split("oracle.", 1)[1].split("-server", 1)[0]
    config["additional_user_agent"] = f"{user_agent_name}/{__version__}"

    private_key = oci.signer.load_private_key_from_file(config["key_file"])
    token_file = os.path.expanduser(config["security_token_file"])
    token = None
    with open(token_file, "r") as f:
        token = f.read()
    signer = oci.auth.signers.SecurityTokenSigner(token, private_key)
    return oci.object_storage.ObjectStorageClient(config, signer=signer)


# Object storage namespace
def get_object_storage_namespace(compartment_id: str):
    object_storage_client = get_object_storage_client()
    namespace = object_storage_client.get_namespace(compartment_id=compartment_id)
    return namespace.data


@mcp.tool(description="Get the object storage namespace for the tenancy")
def get_namespace(
    compartment_id: Annotated[
        str,
        "The OCID of the compartment."
        "If compartment id is not provided, use the root compartment id or the tenancy id",
    ],
):
    return get_object_storage_namespace(compartment_id)


# Buckets
@mcp.tool(description="List object storage buckets")
def list_buckets(
    compartment_id: Annotated[
        str,
        "The OCID of the compartment."
        "If compartment id is not provided, use the root compartment id or the tenancy id",
    ],
) -> List[BucketSummary]:
    object_storage_client = get_object_storage_client()
    namespace_name = get_object_storage_namespace(compartment_id)
    buckets = object_storage_client.list_buckets(namespace_name, compartment_id).data
    return [map_bucket_summary(bucket) for bucket in buckets]


@mcp.tool(description="Get details for a specific object storage bucket")
def get_bucket_details(
    bucket_name: Annotated[str, "The name of the bucket"],
    compartment_id: Annotated[
        str,
        "The OCID of the compartment."
        "If compartment id is not provided, use the root compartment id or the tenancy id",
    ],
) -> Bucket:
    object_storage_client = get_object_storage_client()
    namespace_name = get_object_storage_namespace(compartment_id)
    bucket_details = object_storage_client.get_bucket(
        namespace_name,
        bucket_name,
        fields=[
            "approximateSize",
            "approximateCount",
            "autoTiering",
        ],
    ).data

    return map_bucket(bucket_details)


# Objects
@mcp.tool(description="List objects in a given object storage bucket")
def list_objects(
    bucket_name: Annotated[str, "The name of the bucket"],
    compartment_id: Annotated[
        str,
        "The OCID of the compartment."
        "If compartment id is not provided, use the root compartment id or the tenancy id",
    ],
    prefix: Annotated[str, "Optional prefix to filter objects"] = "",
) -> ListObjects:
    object_storage_client = get_object_storage_client()
    namespace_name = get_object_storage_namespace(compartment_id)
    list_objects = object_storage_client.list_objects(
        namespace_name,
        bucket_name,
        prefix=prefix,
        fields="name,size,timeModified,archivalState,storageTier",
    ).data

    objects = [map_object_summary(obj) for obj in list_objects.objects]
    prefixes = list_objects.prefixes if list_objects.prefixes else []
    return ListObjects(objects=objects, prefixes=prefixes)


@mcp.tool(description="List object versions in a given object storage bucket")
def list_object_versions(
    bucket_name: Annotated[str, "The name of the bucket"],
    compartment_id: Annotated[
        str,
        "The OCID of the compartment."
        "If compartment id is not provided, use the root compartment id or the tenancy id",
    ],
    prefix: Annotated[str, "Optional prefix to filter object versions"] = "",
) -> ObjectVersionCollection:
    object_storage_client = get_object_storage_client()
    namespace_name = get_object_storage_namespace(compartment_id)
    list_object_versions = object_storage_client.list_object_versions(
        namespace_name,
        bucket_name,
        prefix=prefix,
        limit=25,
        fields="timeModified",
    ).data

    versioned_objects = [
        map_object_version_summary(obj) for obj in list_object_versions.items
    ]
    prefixes = list_object_versions.prefixes if list_object_versions.prefixes else []
    return ObjectVersionCollection(items=versioned_objects, prefixes=prefixes)


@mcp.tool(description="Get a specific object from an object storage bucket")
def get_object(
    bucket_name: Annotated[str, "The name of the bucket"],
    compartment_id: Annotated[
        str,
        "The OCID of the compartment."
        "If compartment id is not provided, use the root compartment id or the tenancy id",
    ],
    object_name: Annotated[str, "The name of the object"],
    version_id: Annotated[str, "Optional version ID of the object"] = "",
) -> ObjectSummary:
    object_storage_client = get_object_storage_client()
    namespace_name = get_object_storage_namespace(compartment_id)
    obj = object_storage_client.get_object(
        namespace_name,
        bucket_name,
        object_name,
        version_id=version_id,
    ).data

    return map_object_summary(obj)


@mcp.tool(description="Upload an object to an object storage bucket")
def upload_object(
    bucket_name: Annotated[str, "The name of the bucket"],
    compartment_id: Annotated[
        str,
        "The OCID of the compartment."
        "If compartment id is not provided, use the root compartment id or the tenancy id",
    ],
    file_path: Annotated[str, "The path to the file to upload"],
    object_name: Annotated[
        str,
        "Optional name of the object to upload"
        "If the object name is not provided, use the file name as the object name",
    ] = "",
):
    object_storage_client = get_object_storage_client()
    namespace_name = get_object_storage_namespace(compartment_id)
    logger.info("Got Namespace: %s", namespace_name)
    logger.info("Checking file at path: %s", file_path)
    try:
        with open(file_path, "rb") as file:
            object_storage_client.put_object(
                namespace_name, bucket_name, object_name, file
            )
        return {"message": "Object uploaded successfully"}
    except Exception as e:
        return {"error": str(e)}


def main():

    host = os.getenv("ORACLE_MCP_HOST")
    port = os.getenv("ORACLE_MCP_PORT")

    if host and port:
        mcp.run(transport="http", host=host, port=int(port))
    else:
        mcp.run()


if __name__ == "__main__":
    main()



"""
Copyright (c) 2025, Oracle and/or its affiliates.
Licensed under the Universal Permissive License v1.0 as shown at
https://oss.oracle.com/licenses/upl.
"""

from datetime import datetime
from typing import Any, Dict, List, Optional

import oci.object_storage.models
from pydantic import BaseModel, Field


class Bucket(BaseModel):
    namespace: Optional[str] = Field(
        None,
        description=("The Object Storage namespace in which the bucket " "resides."),
    )
    name: Optional[str] = Field(
        None,
        description="The name of the bucket.",
    )
    compartment_id: Optional[str] = Field(
        None,
        description=("The compartment ID in which the bucket is " "authorized."),
    )
    metadata: Optional[Dict[str, str]] = Field(
        None,
        description=("Arbitrary string keys and values for user-defined " "metadata."),
    )
    created_by: Optional[str] = Field(
        None,
        description="The OCID of the user who created the bucket.",
    )
    time_created: Optional[datetime] = Field(
        None,
        description="The date and time the bucket was created.",
    )
    etag: Optional[str] = Field(
        None,
        description="The entity tag (ETag) for the bucket.",
    )
    public_access_type: Optional[str] = Field(
        None,
        description=("The type of public access enabled on this bucket."),
    )
    storage_tier: Optional[str] = Field(
        None,
        description="The storage tier type assigned to the bucket.",
    )
    object_events_enabled: Optional[bool] = Field(
        None,
        description=(
            "Whether or not events are emitted for object state "
            "changes in this bucket."
        ),
    )
    freeform_tags: Optional[Dict[str, str]] = Field(
        None,
        description="Free-form tags for this resource.",
    )
    defined_tags: Optional[Dict[str, Dict[str, object]]] = Field(
        None,
        description="Defined tags for this resource.",
    )
    kms_key_id: Optional[str] = Field(
        None,
        description=(
            "The OCID of a master encryption key used to call "
            "the Key Management service."
        ),
    )
    object_lifecycle_policy_etag: Optional[str] = Field(
        None,
        description=(
            "The entity tag (ETag) for the live object lifecycle "
            "policy on the bucket."
        ),
    )
    approximate_count: Optional[int] = Field(
        None,
        description="The approximate number of objects in the bucket.",
    )
    approximate_size: Optional[int] = Field(
        None,
        description=(
            "The approximate total size in bytes of all objects " "in the bucket."
        ),
    )
    replication_enabled: Optional[bool] = Field(
        None,
        description="Whether or not this bucket is a replication source.",
    )
    is_read_only: Optional[bool] = Field(
        None,
        description="Whether or not this bucket is read only.",
    )
    id: Optional[str] = Field(
        None,
        description="The OCID of the bucket.",
    )
    versioning: Optional[str] = Field(
        None,
        description="The versioning status on the bucket.",
    )
    auto_tiering: Optional[str] = Field(
        None,
        description="The auto tiering status on the bucket.",
    )

    class Config:
        extra = "allow"
        arbitrary_types_allowed = True
        json_encoders = {
            datetime: lambda dt: dt.isoformat(),
        }


class NamespaceMetadata(BaseModel):
    """
    Pydantic model mirroring the fields of oci.object_storage.models.NamespaceMetadata.
    """

    namespace: str = Field(
        ...,
        description=("The Object Storage namespace to which the metadata " "belongs."),
    )
    default_s3_compartment_id: Optional[str] = Field(
        None,
        description=(
            "If set, specifies the default compartment assignment "
            "for the Amazon S3 Compatibility API."
        ),
    )
    default_swift_compartment_id: Optional[str] = Field(
        None,
        description=(
            "If set, specifies the default compartment assignment " "for the Swift API."
        ),
    )

    class Config:
        extra = "forbid"


class CreateBucketDetails(BaseModel):
    """
    Pydantic model mirroring the fields of
    oci.object_storage.models.CreateBucketDetails.
    """

    name: str = Field(
        ...,
        description="The name of the bucket.",
    )
    compartment_id: str = Field(
        ...,
        description=("The ID of the compartment in which to create the " "bucket."),
    )
    metadata: Optional[Dict[str, str]] = Field(
        None,
        description=(
            "Arbitrary string, up to 4KB, of keys and values for "
            "user-defined metadata."
        ),
    )
    public_access_type: Optional[str] = Field(
        None,
        description=("The type of public access enabled on this bucket."),
    )
    storage_tier: Optional[str] = Field(
        None,
        description="The type of storage tier of this bucket.",
    )
    object_events_enabled: Optional[bool] = Field(
        None,
        description=(
            "Whether or not events are emitted for object state "
            "changes in this bucket."
        ),
    )
    freeform_tags: Optional[Dict[str, str]] = Field(
        None,
        description="Free-form tags for this resource.",
    )
    defined_tags: Optional[Dict[str, Dict[str, Any]]] = Field(
        None,
        description="Defined tags for this resource.",
    )
    kms_key_id: Optional[str] = Field(
        None,
        description=(
            "The OCID of a master encryption key used to call "
            "the Key Management service."
        ),
    )
    versioning: Optional[str] = Field(
        None,
        description="The versioning status on the bucket.",
    )
    auto_tiering: Optional[str] = Field(
        None,
        description="The auto tiering status on the bucket.",
    )

    class Config:
        extra = "forbid"


class BucketSummary(BaseModel):
    namespace: Optional[str] = Field(
        None,
        description=("The Object Storage namespace in which the bucket " "lives."),
    )
    name: Optional[str] = Field(
        None,
        description="The name of the bucket.",
    )
    compartment_id: Optional[str] = Field(
        None,
        description=("The compartment ID in which the bucket is " "authorized."),
    )
    created_by: Optional[str] = Field(
        None,
        description="The OCID of the user who created the bucket.",
    )
    time_created: Optional[datetime] = Field(
        None,
        description="The date and time the bucket was created.",
    )
    etag: Optional[str] = Field(
        None,
        description="The entity tag (ETag) for the bucket.",
    )
    freeform_tags: Optional[Dict[str, str]] = Field(
        None,
        description="Free-form tags for this resource.",
    )
    defined_tags: Optional[Dict[str, Dict[str, object]]] = Field(
        None,
        description="Defined tags for this resource.",
    )

    class Config:
        extra = "allow"
        arbitrary_types_allowed = True
        json_encoders = {
            datetime: lambda dt: dt.isoformat(),
        }


class ObjectSummary(BaseModel):
    name: Optional[str] = Field(
        None,
        description="The name of the object.",
    )
    size: Optional[int] = Field(
        None,
        description="Size of the object in bytes.",
    )
    md5: Optional[str] = Field(
        None,
        description="Base64-encoded MD5 hash of the object data.",
    )
    time_created: Optional[datetime] = Field(
        None,
        description="The date and time the object was created.",
    )
    etag: Optional[str] = Field(
        None,
        description="The current entity tag (ETag) for the object.",
    )
    storage_tier: Optional[str] = Field(
        None,
        description="The storage tier that the object is stored in.",
    )
    archival_state: Optional[str] = Field(
        None,
        description="Archival state of an object.",
    )
    time_modified: Optional[datetime] = Field(
        None,
        description="The date and time the object was modified.",
    )

    class Config:
        extra = "allow"
        arbitrary_types_allowed = True
        json_encoders = {
            datetime: lambda dt: dt.isoformat(),
        }


class ListObjects(BaseModel):
    objects: Optional[List[ObjectSummary]] = Field(
        None,
        description="An array of object summaries.",
    )
    prefixes: Optional[List[str]] = Field(
        None,
        description=(
            "Prefixes that are common to the results returned "
            "by the request if the request specified a delimiter."
        ),
    )
    next_start_with: Optional[str] = Field(
        None,
        description=(
            "The name of the object to use in the `start` "
            "parameter to obtain the next page of a truncated "
            "ListObjects response."
        ),
    )

    class Config:
        extra = "allow"
        arbitrary_types_allowed = True


class ObjectVersionSummary(BaseModel):
    name: Optional[str] = Field(
        None,
        description="The name of the object.",
    )
    size: Optional[int] = Field(
        None,
        description="Size of the object in bytes.",
    )
    md5: Optional[str] = Field(
        None,
        description="Base64-encoded MD5 hash of the object data.",
    )
    time_created: Optional[datetime] = Field(
        None,
        description="The date and time the object was created.",
    )
    time_modified: Optional[datetime] = Field(
        None,
        description="The date and time the object was modified.",
    )
    etag: Optional[str] = Field(
        None,
        description="The current entity tag (ETag) for the object.",
    )
    storage_tier: Optional[str] = Field(
        None,
        description="The storage tier that the object is stored in.",
    )
    archival_state: Optional[str] = Field(
        None,
        description="Archival state of an object.",
    )
    version_id: Optional[str] = Field(
        None,
        description="VersionId of the object.",
    )
    is_delete_marker: Optional[bool] = Field(
        None,
        description="This flag will indicate if the version is deleted or not.",
    )

    class Config:
        extra = "allow"
        arbitrary_types_allowed = True
        json_encoders = {
            datetime: lambda dt: dt.isoformat(),
        }


class ObjectVersionCollection(BaseModel):
    """
    Pydantic model mirroring the fields of oci.object_storage.models.ObjectVersionCollection.
    """

    items: List[ObjectVersionSummary] = Field(
        ..., description="An array of object version summaries."
    )
    prefixes: Optional[List[str]] = Field(
        None,
        description=(
            "Prefixes that are common to the results returned by the request if the request "
            "specified a delimiter."
        ),
    )

    class Config:
        extra = "forbid"


def map_object_summary(obj: oci.object_storage.models.ObjectSummary) -> ObjectSummary:
    """
    Convert an oci.object_storage.models.ObjectSummary to an
    oracle.oci_object_storage_mcp_server.models.ObjectSummary, including all nested types
    """
    return ObjectSummary(
        name=obj.name,
        size=getattr(obj, "size", None),
        time_created=getattr(obj, "time_created", None),
        etag=getattr(obj, "etag", None),
        storage_tier=getattr(obj, "storage_tier", None),
        archival_state=getattr(obj, "archival_state", None),
        time_modified=getattr(obj, "time_modified", None),
    )


def map_bucket_summary(
    bucket: oci.object_storage.models.BucketSummary,
) -> BucketSummary:
    """
    Convert an oci.object_storage.models.BucketSummary to an
    oracle.oci_object_storage_mcp_server.models.BucketSummary, including all nested types
    """
    return BucketSummary(
        namespace=getattr(bucket, "namespace", None),
        name=getattr(bucket, "name", None),
        compartment_id=getattr(bucket, "compartment_id", None),
        created_by=getattr(bucket, "created_by", None),
        time_created=getattr(bucket, "time_created", None),
        etag=getattr(bucket, "etag", None),
        freeform_tags=getattr(bucket, "freeform_tags", None),
        defined_tags=getattr(bucket, "defined_tags", None),
    )


def map_bucket(bucket: oci.object_storage.models.Bucket) -> Bucket:
    """
    Convert an oci.object_storage.models.Bucket to an oracle.oci_object_storage_mcp_server.models.Bucket,
    including all nested types
    """
    return Bucket(
        namespace=getattr(bucket, "namespace", None),
        name=getattr(bucket, "name", None),
        compartment_id=getattr(bucket, "compartment_id", None),
        metadata=getattr(bucket, "metadata", None),
        created_by=getattr(bucket, "created_by", None),
        time_created=getattr(bucket, "time_created", None),
        etag=getattr(bucket, "etag", None),
        public_access_type=getattr(bucket, "public_access_type", None),
        storage_tier=getattr(bucket, "storage_tier", None),
        object_events_enabled=getattr(bucket, "object_events_enabled", None),
        freeform_tags=getattr(bucket, "freeform_tags", None),
        defined_tags=getattr(bucket, "defined_tags", None),
        kms_key_id=getattr(bucket, "kms_key_id", None),
        object_lifecycle_policy_etag=getattr(
            bucket, "object_lifecycle_policy_etag", None
        ),
        approximate_count=getattr(bucket, "approximate_count", None),
        approximate_size=getattr(bucket, "approximate_size", None),
        replication_enabled=getattr(bucket, "replication_enabled", None),
        is_read_only=getattr(bucket, "is_read_only", None),
        id=getattr(bucket, "id", None),
        versioning=getattr(bucket, "versioning", None),
        auto_tiering=getattr(bucket, "auto_tiering", None),
    )


def map_object_version_summary(obj: oci.object_storage.models.ObjectVersionSummary):
    return ObjectVersionSummary(
        name=obj.name,
        size=getattr(obj, "size", None),
        md5=getattr(obj, "md5", None),
        time_created=getattr(obj, "time_created", None),
        time_modified=getattr(obj, "time_modified", None),
        etag=getattr(obj, "etag", None),
        storage_tier=getattr(obj, "storage_tier", None),
        archival_state=getattr(obj, "archival_state", None),
        version_id=getattr(obj, "version_id", None),
        is_delete_marker=getattr(obj, "is_delete_marker", None),
    )



"""
Copyright (c) 2026, Oracle and/or its affiliates.
Licensed under the Universal Permissive License v1.0 as shown at
https://oss.oracle.com/licenses/upl.
"""

from datetime import datetime, timezone
from types import SimpleNamespace

import pytest
from oracle.oci_object_storage_mcp_server.models import (
    Bucket,
    BucketSummary,
    CreateBucketDetails,
    ListObjects,
    NamespaceMetadata,
    ObjectSummary,
    ObjectVersionCollection,
    ObjectVersionSummary,
    map_bucket,
    map_bucket_summary,
    map_object_summary,
    map_object_version_summary,
)
from pydantic import ValidationError


class TestModelMapping:
    def test_map_object_summary_from_oci_like_object(self):
        # Use a simple namespace to emulate the OCI SDK object with attributes
        src = SimpleNamespace(
            name="obj1",
            size=123,
            time_created=datetime(2021, 1, 1, tzinfo=timezone.utc),
            etag="etag-1",
            storage_tier="STANDARD",
            archival_state="ARCHIVED",
            time_modified=datetime(2021, 1, 2, tzinfo=timezone.utc),
        )

        mapped = map_object_summary(src)
        assert isinstance(mapped, ObjectSummary)
        assert mapped.name == "obj1"
        assert mapped.size == 123
        assert mapped.storage_tier == "STANDARD"
        # json encodes datetimes using isoformat
        json_text = mapped.json()
        assert "2021-01-01T00:00:00" in json_text
        assert "2021-01-02T00:00:00" in json_text

    def test_map_bucket_summary_from_oci_like_object(self):
        src = SimpleNamespace(
            namespace="ns1",
            name="bucket1",
            compartment_id="ocid1.compartment.oc1..xyz",
            created_by="ocid1.user.oc1..abc",
            time_created=datetime(2021, 1, 1, tzinfo=timezone.utc),
            etag="etag-bkt",
            freeform_tags={"env": "dev"},
            defined_tags={"ns": {"k": "v"}},
        )

        mapped = map_bucket_summary(src)
        assert isinstance(mapped, BucketSummary)
        assert mapped.name == "bucket1"
        assert mapped.namespace == "ns1"
        assert mapped.freeform_tags == {"env": "dev"}

    def test_map_bucket_from_oci_like_object(self):
        src = SimpleNamespace(
            namespace="ns1",
            name="bucket1",
            compartment_id="ocid1.compartment.oc1..xyz",
            metadata={"a": "b"},
            created_by="ocid1.user.oc1..abc",
            time_created=datetime(2021, 1, 1, tzinfo=timezone.utc),
            etag="etag-bkt",
            public_access_type="NoPublicAccess",
            storage_tier="Standard",
            object_events_enabled=False,
            freeform_tags={"env": "dev"},
            defined_tags={"ns": {"k": "v"}},
            kms_key_id="ocid1.key.oc1..key",
            object_lifecycle_policy_etag="olp-etag",
            approximate_count=10,
            approximate_size=2048,
            replication_enabled=False,
            is_read_only=False,
            id="ocid1.bucket.oc1..bucket",
            versioning="Enabled",
            auto_tiering="InfrequentAccess",
        )

        mapped = map_bucket(src)
        assert isinstance(mapped, Bucket)
        assert mapped.name == "bucket1"
        assert mapped.approximate_size == 2048
        assert mapped.versioning == "Enabled"

    def test_map_object_version_summary_from_oci_like_object(self):
        src = SimpleNamespace(
            name="obj1",
            size=123,
            md5="md5==",
            time_created=datetime(2021, 1, 1, tzinfo=timezone.utc),
            time_modified=datetime(2021, 1, 2, tzinfo=timezone.utc),
            etag="etag-1",
            storage_tier="STANDARD",
            archival_state="ARCHIVED",
            version_id="ver-1",
            is_delete_marker=False,
        )

        mapped = map_object_version_summary(src)
        assert isinstance(mapped, ObjectVersionSummary)
        assert mapped.version_id == "ver-1"
        assert mapped.is_delete_marker is False


class TestModelValidation:
    def test_namespace_metadata_forbids_extra_fields(self):
        with pytest.raises(ValidationError):
            NamespaceMetadata(namespace="ns", unknown="x")

    def test_create_bucket_details_forbids_extra_fields(self):
        with pytest.raises(ValidationError):
            CreateBucketDetails(name="b", compartment_id="c", unknown="x")

    def test_object_version_collection_forbids_extra_fields(self):
        items = [ObjectVersionSummary(name="o")]  # minimal valid item
        with pytest.raises(ValidationError):
            ObjectVersionCollection(items=items, unknown="x")

    def test_list_objects_allows_extra_fields(self):
        lo = ListObjects(objects=[ObjectSummary(name="a")], prefixes=["p"], extra=1)
        # extra field should be preserved due to extra="allow"
        assert getattr(lo, "extra") == 1

    def test_bucket_model_allows_extra_fields(self):
        b = Bucket(name="bkt", extra_field="ok")
        assert getattr(b, "extra_field") == "ok"
